@@ -42,7 +42,7 @@ fn create_files_for_contents(contents: String, path: &Path, per_lines: usize) ->
             path.into(),
             vec![LineMatch::lnum(l)],
             vec![(s, e)],
-            contents.clone().into_bytes(),
+            contents.clone(),
         ))
     }
     files
