@@ -1,3 +1,4 @@
+use hgrep::printer::{PrinterOptions, TermColorSupport};
 use std::fs;
 use std::path::Path;
 
@@ -8,7 +9,7 @@ pub fn package_lock_json_path() -> &'static Path {
     let path = Path::new("package-lock.json");
     assert!(
         path.is_file(),
-        "put \"package-lock.json\" file in bench/ directory by `npm install`",
+        "run prepare.bash at hgrep/bench/ directory before running benches",
     );
     path
 }
@@ -22,7 +23,24 @@ pub fn node_modules_path() -> &'static Path {
     let path = Path::new("node_modules");
     assert!(
         path.is_dir(),
-        "put \"node_modules\" directory in bench/ directory by `npm install`",
+        "run prepare.bash at hgrep/bench/ directory before running benches",
     );
     path
 }
+
+pub fn rust_releases_path() -> &'static Path {
+    let path = Path::new("rust_releases.md");
+    assert!(
+        path.is_file(),
+        "run prepare.bash at hgrep/bench/ directory before running benches",
+    );
+    path
+}
+
+pub fn printer_opts() -> PrinterOptions<'static> {
+    PrinterOptions {
+        color_support: TermColorSupport::True,
+        term_width: 80,
+        ..Default::default()
+    }
+}
