@@ -73,15 +73,27 @@ fn command() -> Command {
     #[cfg(all(not(feature = "syntect-printer"), feature = "bat-printer"))]
     const DEFAULT_PRINTER: &str = "bat";
 
+    #[cfg(not(feature = "ripgrep"))]
+    const ABOUT: &str =
+        "hgrep is grep with human-friendly search output. hgrep eats an output of `grep -nH` and prints the matches \
+        with syntax-highlighted code snippets.\n\n\
+        $ grep -nH pattern -R . | hgrep\n\n\
+        The default options can be customized with HGREP_DEFAULT_OPTS environment variable. \
+        For more details, visit https://github.com/rhysd/hgrep#readme";
+    #[cfg(feature = "ripgrep")]
+    const ABOUT: &str =
+        "hgrep is grep with human-friendly search output.\n\n\
+        hgrep eats an output of `grep -nH` and prints the matches with syntax-highlighted code snippets.\n\n\
+        $ grep -nH pattern -R . | hgrep\n\n\
+        hgrep has its builtin subset of ripgrep, whose search output and performance are better than reading \
+        the output from `grep -nH`.\n\n\
+        $ hgrep pattern\n\n\
+        The default options can be customized with HGREP_DEFAULT_OPTS environment variable. \
+        For more details, visit https://github.com/rhysd/hgrep#readme";
+
     let cmd = Command::new("hgrep")
         .version(env!("CARGO_PKG_VERSION"))
-        .about(
-            "hgrep is grep with human-friendly search output. hgrep eats an output of `grep -nH` and prints the matches \
-            with syntax-highlighted code snippets.\n\n\
-            $ grep -nH pattern -R . | hgrep\n\n\
-            The default options can be customized with HGREP_DEFAULT_OPTS environment variable. \
-            For more details, visit https://github.com/rhysd/hgrep#readme"
-        )
+        .about(ABOUT)
         .no_binary_name(true)
         .args_override_self(true)
         .arg(
@@ -171,6 +183,13 @@ fn command() -> Command {
                 .long("first-only")
                 .action(ArgAction::SetTrue)
                 .help("Show only the first code snippet per file")
+        ).arg(
+            Arg::new("encoding")
+                .short('E')
+                .long("encoding")
+                .num_args(1)
+                .value_name("ENCODING")
+                .help("Specify the text encoding that hgrep will use on all files printed like 'sjis'")
         )
         .arg(
             Arg::new("generate-completion-script")
@@ -215,17 +234,6 @@ fn command() -> Command {
 
     #[cfg(feature = "ripgrep")]
     let cmd = cmd
-            .about(
-                "hgrep is grep with human-friendly search output.\n\n\
-                hgrep eats an output of `grep -nH` and prints the matches with syntax-highlighted code snippets.\n\n\
-                $ grep -nH pattern -R . | hgrep\n\n\
-                hgrep has its builtin subset of ripgrep, whose search output and performance are better than reading \
-                the output from `grep -nH`.\n\n\
-                $ hgrep pattern\n\n\
-                The default options can be customized with HGREP_DEFAULT_OPTS environment variable. \
-                For more details, visit https://github.com/rhysd/hgrep#readme"
-            )
-            .override_usage("hgrep [FLAGS] [OPTIONS] [PATTERN [PATH...]]")
             .arg(
                 Arg::new("no-ignore")
                     .long("no-ignore")
@@ -542,6 +550,10 @@ fn build_ripgrep_config(
         _ => anyhow::bail!("-u or --unrestricted cannot be repeated more than twice. Try -uu to search every text file"),
     }
 
+    if let Some(encoding) = matches.get_one::<String>("encoding") {
+        config.encoding(encoding);
+    }
+
     Ok(config)
 }
 
@@ -725,6 +737,8 @@ fn run(matches: ArgMatches) -> Result<bool> {
         unreachable!();
     }
 
+    let encoding = matches.get_one::<String>("encoding").map(String::as_str);
+
     #[cfg(feature = "syntect-printer")]
     if printer_kind == PrinterKind::Syntect {
         use hgrep::printer::Printer;
@@ -732,7 +746,7 @@ fn run(matches: ArgMatches) -> Result<bool> {
         let printer = SyntectPrinter::with_stdout(printer_opts)?;
         return io::BufReader::new(io::stdin())
             .grep_lines()
-            .chunks_per_file(min_context, max_context)
+            .chunks_per_file(min_context, max_context, encoding)?
             .par_bridge()
             .map(|file| {
                 printer.print(file?)?;
@@ -748,7 +762,7 @@ fn run(matches: ArgMatches) -> Result<bool> {
         let stdin = io::stdin();
         for f in io::BufReader::new(stdin.lock())
             .grep_lines()
-            .chunks_per_file(min_context, max_context)
+            .chunks_per_file(min_context, max_context, encoding)?
         {
             printer.print(f?)?;
             found = true;
@@ -855,6 +869,7 @@ mod tests {
         snapshot_test!(unrestricted_once, ["-u"]);
         snapshot_test!(unrestricted_twice, ["-u", "-u"]);
         snapshot_test!(unrestricted_twice_in_single_flag, ["-uu"]);
+        snapshot_test!(encoding, ["--encoding", "sjis"]);
         snapshot_test!(
             all_printer_opts_before_args,
             [
@@ -1067,6 +1082,7 @@ mod tests {
         snapshot_test!(max_filesize, ["--max-filesize", "100M"]);
         snapshot_test!(unrestricted_once, ["-u"]);
         snapshot_test!(unrestricted_twice, ["-u", "-u"]);
+        snapshot_test!(encoding, ["--encoding", "sjis"]);
 
         macro_rules! snapshot_error_test {
             ($name:ident, $args:expr) => {
