@@ -1,12 +1,10 @@
 use crate::broken_pipe::IgnoreBrokenPipe as _;
-use crate::chunk::{File, Line};
+use crate::chunk::{File, LinesInclusive};
 use crate::printer::{Printer, PrinterOptions, TermColorSupport, TextWrapMode};
 use ansi_colours::ansi256_from_rgb;
 use anyhow::Result;
 use flate2::read::ZlibDecoder;
-use memchr::{memchr_iter, Memchr};
 use std::cmp;
-use std::ffi::OsStr;
 use std::io::{self, Stdout, StdoutLock, Write};
 use std::ops::{Deref, DerefMut};
 use std::path::Path;
@@ -608,43 +606,6 @@ impl<'a> LineHighlighter<'a> {
     }
 }
 
-// Like chunk::Lines, but includes newlines
-struct LinesInclusive<'a> {
-    lnum: usize,
-    prev: usize,
-    buf: &'a [u8],
-    iter: Memchr<'a>,
-}
-impl<'a> LinesInclusive<'a> {
-    pub fn new(buf: &'a [u8]) -> Self {
-        Self {
-            lnum: 1,
-            prev: 0,
-            buf,
-            iter: memchr_iter(b'\n', buf),
-        }
-    }
-}
-impl<'a> Iterator for LinesInclusive<'a> {
-    type Item = Line<'a>;
-    fn next(&mut self) -> Option<Self::Item> {
-        if let Some(idx) = self.iter.next() {
-            let lnum = self.lnum;
-            let end = idx + 1;
-            let line = &self.buf[self.prev..end];
-            self.prev = end;
-            self.lnum += 1;
-            Some(Line(line, lnum as u64))
-        } else if self.prev == self.buf.len() {
-            None
-        } else {
-            let line = &self.buf[self.prev..];
-            self.prev = self.buf.len();
-            Some(Line(line, self.lnum as u64))
-        }
-    }
-}
-
 // Drawer is responsible for one-time screen drawing
 struct Drawer<'file, W: Write> {
     grid: bool,
@@ -878,10 +839,10 @@ impl<'file, W: Write> Drawer<'file, W> {
         let mut chunks = file.chunks.iter();
         let mut chunk = chunks.next().unwrap(); // OK since chunks is not empty
 
-        for Line(bytes, lnum) in LinesInclusive::new(&file.contents) {
+        for (line, lnum) in LinesInclusive::new(&file.contents) {
             let (start, end) = *chunk;
             if lnum < start {
-                hl.skip_line(String::from_utf8_lossy(bytes).as_ref())?; // Discard parsed result
+                hl.skip_line(line)?; // Discard parsed result
                 continue;
             }
             if start <= lnum && lnum <= end {
@@ -892,10 +853,9 @@ impl<'file, W: Write> Drawer<'file, W> {
                     }
                     _ => None,
                 };
-                let line = String::from_utf8_lossy(bytes);
                 // Collect to `Vec` rather than handing HighlightIterator as-is. HighlightIterator takes ownership of Highlighter
                 // while the iteration. When the highlighter is stored in `self`, it means the iterator takes ownership of `self`.
-                self.draw_line(hl.highlight(line.as_ref())?, lnum, regions)?;
+                self.draw_line(hl.highlight(line)?, lnum, regions)?;
 
                 if lnum == end {
                     if self.first_only {
@@ -1036,84 +996,83 @@ impl<'main, W> SyntectPrinter<'main, W> {
         &self.themes.themes[name]
     }
 
-    fn find_syntax(&self, path: &Path) -> Result<&SyntaxReference> {
-        // Find from file extension
-        let name = match path.extension().and_then(OsStr::to_str) {
-            Some("fs") => Some("F#"),
-            Some("h") => Some("C++"),
-            Some("pac") => Some("JavaScript (Babel)"),
-            Some("nse") => Some("Lua"),
-            Some(
+    fn find_syntax(&self, file: &File) -> &SyntaxReference {
+        let extension = file.path.extension();
+        let file_name = file.path.file_name();
+
+        let name =
+            // Find from file extension
+            extension.and_then(|e| match e.to_str()? {
+                "fs" => Some("F#"),
+                "h" => Some("C++"),
+                "pac" => Some("JavaScript (Babel)"),
+                "nse" => Some("Lua"),
                 "automount" | "device" | "dnssd" | "link" | "mount" | "netdev" | "network"
-                | "nspawn" | "path" | "service" | "scope" | "slice" | "socket" | "swap" | "target"
-                | "timer",
-            ) => Some("INI"),
-            Some("sarif" | "jsonl") => Some("JSON"),
-            Some("ron") => Some("Rust"),
-            _ => None,
-        };
+                | "nspawn" | "path" | "service" | "scope" | "slice" | "socket" | "swap"
+                | "target" | "timer" => Some("INI"),
+                "sarif" | "jsonl" => Some("JSON"),
+                "ron" => Some("Rust"),
+                _ => None,
+            })
+            // Find from file name
+            .or_else(|| match file_name?.to_str()? {
+                ".clang-format" | "fish_history" => Some("YAML"),
+                "nginx.conf" | "mime.types" => Some("nginx"),
+                "httpd.conf" => Some("Apache Conf"),
+                "Containerfile" => Some("Dockerfile"),
+                _ => None,
+            })
+            // Find from file path
+            .or_else(|| {
+                #[cfg(not(windows))]
+                const GIT_CONFIG: &str = "/git/config";
+                #[cfg(windows)]
+                const GIT_CONFIG: &str = "\\git\\config";
+                #[cfg(not(windows))]
+                const GIT_IGNORE: &str = "/git/ignore";
+                #[cfg(windows)]
+                const GIT_IGNORE: &str = "\\git\\ignore";
+                #[cfg(not(windows))]
+                const GIT_ATTRIBUTES: &str = "/git/attributes";
+                #[cfg(windows)]
+                const GIT_ATTRIBUTES: &str = "\\git\\attributes";
+                #[cfg(not(windows))]
+                const SSH_CONFIG: &str = "/.ssh/config";
+                #[cfg(windows)]
+                const SSH_CONFIG: &str = "\\.ssh\\config";
 
-        // Find from file name
-        let name = name.or_else(|| match path.file_name().and_then(OsStr::to_str) {
-            Some(".clang-format" | "fish_history") => Some("YAML"),
-            Some("nginx.conf" | "mime.types") => Some("nginx"),
-            Some("httpd.conf") => Some("Apache Conf"),
-            Some("Containerfile") => Some("Dockerfile"),
-            _ => None,
-        });
+                let path = file.path.to_str()?;
+                if path.ends_with(GIT_CONFIG) {
+                    return Some("Git Config");
+                }
+                if path.ends_with(GIT_IGNORE) {
+                    return Some("Git Ignore");
+                }
+                if path.ends_with(GIT_ATTRIBUTES) {
+                    return Some("Git Attributes");
+                }
+                if path.ends_with(SSH_CONFIG) {
+                    return Some("SSH Config");
+                }
+                #[cfg(not(windows))]
+                if path == "/etc/profile" {
+                    return Some("Bourne Again Shell (bash)");
+                }
+                #[cfg(not(windows))]
+                if path.starts_with("/var/spool/mail/") || path.starts_with("/var/mail/") {
+                    return Some("Email");
+                }
+                None
+            });
 
-        // Find from file path
-        let name = name.or_else(|| {
-            #[cfg(not(windows))]
-            const GIT_CONFIG: &str = "/git/config";
-            #[cfg(windows)]
-            const GIT_CONFIG: &str = "\\git\\config";
-            #[cfg(not(windows))]
-            const GIT_IGNORE: &str = "/git/ignore";
-            #[cfg(windows)]
-            const GIT_IGNORE: &str = "\\git\\ignore";
-            #[cfg(not(windows))]
-            const GIT_ATTRIBUTES: &str = "/git/attributes";
-            #[cfg(windows)]
-            const GIT_ATTRIBUTES: &str = "\\git\\attributes";
-            #[cfg(not(windows))]
-            const SSH_CONFIG: &str = "/.ssh/config";
-            #[cfg(windows)]
-            const SSH_CONFIG: &str = "\\.ssh\\config";
-
-            let path = path.to_str()?;
-            if path.ends_with(GIT_CONFIG) {
-                return Some("Git Config");
-            }
-            if path.ends_with(GIT_IGNORE) {
-                return Some("Git Ignore");
-            }
-            if path.ends_with(GIT_ATTRIBUTES) {
-                return Some("Git Attributes");
-            }
-            if path.ends_with(SSH_CONFIG) {
-                return Some("SSH Config");
-            }
-            #[cfg(not(windows))]
-            if path == "/etc/profile" {
-                return Some("Bourne Again Shell (bash)");
-            }
-            #[cfg(not(windows))]
-            if path.starts_with("/var/spool/mail/") || path.starts_with("/var/mail/") {
-                return Some("Email");
-            }
-
-            None
-        });
-
-        if let Some(syntax) = name.and_then(|n| self.syntaxes.find_syntax_by_name(n)) {
-            return Ok(syntax);
-        }
-
-        Ok(self
-            .syntaxes
-            .find_syntax_for_file(path)?
-            .unwrap_or_else(|| self.syntaxes.find_syntax_plain_text()))
+        // Avoid `SyntaxSet::find_syntax_for_file` since it opens the file and reads the first line.
+        // (That's why `SyntaxSet::find_syntax_for_file` returns `io::Result`).
+        // It is redundant since we already read the file content into `File` struct.
+        name.and_then(|n| self.syntaxes.find_syntax_by_name(n))
+            .or_else(|| self.syntaxes.find_syntax_by_extension(extension?.to_str()?))
+            .or_else(|| self.syntaxes.find_syntax_by_extension(file_name?.to_str()?))
+            .or_else(|| self.syntaxes.find_syntax_by_first_line(file.first_line()))
+            .unwrap_or_else(|| self.syntaxes.find_syntax_plain_text())
     }
 }
 
@@ -1125,7 +1084,7 @@ impl<'main, W: WriteOnLocked> Printer for SyntectPrinter<'main, W> {
 
         let mut buf = vec![];
         let theme = self.theme();
-        let syntax = self.find_syntax(&file.path)?;
+        let syntax = self.find_syntax(&file);
 
         let hl = LineHighlighter::new(syntax, theme, &self.syntaxes);
         Drawer::new(&mut buf, &self.opts, theme, &file.chunks).draw_file(&file, hl)?;
@@ -1222,7 +1181,7 @@ mod tests {
                 lmats.extend(ls);
                 chunks.push(c);
             }
-            File::new(path, lmats, chunks, contents.into_bytes())
+            File::new(path, lmats, chunks, contents)
         }
 
         #[cfg(not(windows))]
@@ -1270,7 +1229,7 @@ mod tests {
         }
 
         fn run_uitest(file: File, expected_file: PathBuf, f: fn(&mut PrinterOptions<'_>) -> ()) {
-            let stdout = DummyStdout(RefCell::new(vec![]));
+            let stdout = DummyStdout::default();
             let mut opts = PrinterOptions {
                 term_width: 80,
                 color_support: TermColorSupport::True,
@@ -1537,7 +1496,7 @@ mod tests {
         let readme = PathBuf::from(file);
         let lmats = vec![LineMatch::lnum(3)];
         let chunks = vec![(1, 6)];
-        let contents = fs::read(&readme).unwrap();
+        let contents = fs::read_to_string(&readme).unwrap();
         File::new(readme, lmats, chunks, contents)
     }
 
@@ -1579,9 +1538,9 @@ mod tests {
 
     #[test]
     fn test_print_nothing() {
-        let file = File::new(PathBuf::from("x.txt"), vec![], vec![], vec![]);
+        let file = File::new(PathBuf::from("x.txt"), vec![], vec![], String::new());
         let opts = PrinterOptions::default();
-        let stdout = DummyStdout(RefCell::new(vec![]));
+        let stdout = DummyStdout::default();
         let mut printer = SyntectPrinter::with_assets(ASSETS.clone(), stdout, opts);
         printer.print(file).unwrap();
         let printed = mem::take(printer.writer_mut()).0.into_inner();
@@ -1596,7 +1555,7 @@ mod tests {
     fn test_no_syntax_found() {
         let file = sample_chunk("LICENSE.txt");
         let opts = PrinterOptions::default();
-        let stdout = DummyStdout(RefCell::new(vec![]));
+        let stdout = DummyStdout::default();
         let mut printer = SyntectPrinter::with_assets(ASSETS.clone(), stdout, opts);
         printer.print(file).unwrap();
         let printed = mem::take(printer.writer_mut()).0.into_inner();
@@ -1605,20 +1564,25 @@ mod tests {
 
     #[test]
     fn test_adjacent_regions() {
-        let contents = b"this is test\n";
+        let contents = "this is test\n";
         let ranges = (0..contents.len()).map(|i| (i, i + 1)).collect();
         let lmats = vec![LineMatch {
             line_number: 1,
             ranges,
         }];
         let chunks = vec![(1, 1)];
-        let file = File::new(PathBuf::from("test.txt"), lmats, chunks, contents.to_vec());
+        let file = File::new(
+            PathBuf::from("test.txt"),
+            lmats,
+            chunks,
+            contents.to_string(),
+        );
 
         let opts = PrinterOptions {
             color_support: TermColorSupport::True,
             ..Default::default()
         };
-        let stdout = DummyStdout(RefCell::new(vec![]));
+        let stdout = DummyStdout::default();
         let mut printer = SyntectPrinter::with_assets(ASSETS.clone(), stdout, opts);
         printer.print(file).unwrap();
 
@@ -1660,4 +1624,85 @@ mod tests {
         )
         .unwrap();
     }
+
+    #[test]
+    fn test_find_syntax_from_path() {
+        let tests = [
+            // Additional syntaxes by file extensions
+            ("foo.fs", "F#"),
+            ("/path/to/foo.fs", "F#"),
+            ("foo.h", "C++"),
+            ("foo.ron", "Rust"),
+            // Additional syntaxes by file name
+            ("nginx.conf", "nginx"),
+            ("/path/to/nginx.conf", "nginx"),
+            // Additional syntaxes by file path
+            ("/path/to/git/config", "Git Config"),
+            ("/path/to/git/ignore", "Git Ignore"),
+            ("/path/to/git/attributes", "Git Attributes"),
+            ("/path/to/.ssh/config", "SSH Config"),
+            #[cfg(not(windows))]
+            ("/etc/profile", "Bourne Again Shell (bash)"),
+            #[cfg(not(windows))]
+            ("/var/spool/mail/foo", "Email"),
+            #[cfg(not(windows))]
+            ("/var/mail/foo", "Email"),
+            // From syntax definitions by file extension
+            ("foo.c", "C"),
+            ("/path/to/foo.c", "C"),
+            // From syntax definitions by file name
+            ("Makefile", "Makefile"),
+            ("/path/to/Makefile", "Makefile"),
+            // No syntax found
+            ("foooooooo", "Plain Text"),
+            ("/path/to/foooooooo", "Plain Text"),
+        ];
+
+        let printer = SyntectPrinter::with_assets(
+            ASSETS.clone(),
+            DummyStdout::default(),
+            PrinterOptions::default(),
+        );
+
+        for (path, name) in tests {
+            #[cfg(windows)]
+            let path = {
+                let mut path = path.replace('/', "\\");
+                if path.starts_with('\\') {
+                    path.insert_str(0, "C:");
+                }
+                path
+            };
+            let file = File::new(PathBuf::from(&path), vec![], vec![], String::new());
+            let syntax = printer.find_syntax(&file);
+            assert_eq!(
+                syntax.name, name,
+                "could not find correct syntax from file path {path:?}",
+            );
+        }
+    }
+
+    #[test]
+    fn test_find_syntax_from_first_line() {
+        let tests = [
+            (r#"<?xml version="1.0" encoding="utf-8"?>"#, "XML"),
+            ("#!/bin/bash", "Bourne Again Shell (bash)"),
+            ("hello, world!", "Plain Text"),
+        ];
+
+        let printer = SyntectPrinter::with_assets(
+            ASSETS.clone(),
+            DummyStdout::default(),
+            PrinterOptions::default(),
+        );
+        for (line, name) in tests {
+            let contents = [line, "this is second line", "this is third line"].join("\n");
+            let file = File::new(PathBuf::from("foooooooo"), vec![], vec![], contents);
+            let syntax = printer.find_syntax(&file);
+            assert_eq!(
+                syntax.name, name,
+                "could not find correct syntax from first line {line:?}",
+            );
+        }
+    }
 }
